## =======================================================
##
## Calculates the number of possible sanitation systems (SanSys) given a
## compatible matrix for the 'wet', 'urine', and 'feces' streams.
##
## For details see:
## Maurer et al. (2012), "A compatibility-based procedure designed
##    to generate potential sanitation-system alternatives",
##    Journal of Environmental Management.
##
## contact: max.maurer@eawag.ch
##
## Note: This implementation is only for transitive matrices correct.
##
## January 6, 2012 - Andreas Scheidegger
## =======================================================



## =================================
## 1) import a matrix for every stream

## set path where the csv-files with the compatibility matrices are stored:
setwd("/Users/dorotheespuhler/Dropbox/PHD\ Dropbox/0\ MODELS\ and\ SOFTWARE/R/Compatibility/")
## Note: unlike in MS Windows directories must be separated by "/" and not with "\".

urine <- as.matrix(read.table('urine.csv', sep=';'))
feces <- as.matrix(read.table('feces.csv', sep=';'))
wet <- as.matrix(read.table('wet.csv', sep=';'))


## =================================
## 2) define how to treat 'not full (in)compatible' entries:
## compatible (set <- 1) or incompatible (set <- 0)
set <- 0

urine[urine==0.5] <- set
feces[feces==0.5] <- set
wet[wet==0.5] <- set


## =================================
## 3) define the number of options for each processes

options.urine <- rep(c("interface", "on-site", "conveyance", "centralized", "disposal"),
                     c(5, 2, 3, 2, 3))
## for example: the urine stream has 5 interfaces options, 2 on-site options, 3 conveyance, etc.
options.feces <- rep(c("interface", "on-site", "conveyance", "centralized", "disposal"),
                     c(5, 6, 3, 6, 3))
options.wet <- rep(c("interface", "on-site", "conveyance", "centralized", "disposal"),
                   c(5, 6, 4, 11, 4))


## =================================
## 4) calculate the number of possible SanSys

## number of SanSys if everything is compatible
n.max <- 5 * (2*3*2*3) * (6*3*6*3) * (6*4*11*4)


## function to calculate the number of possible paths
n.path <- function(M, group) {
    proc <- c("interface", "on-site", "conveyance", "centralized", "disposal")
    M.res <- M[group==proc[1],group==proc[1]]
    for(k in 1:4) { M.res <- M.res %*% M[group==proc[k],group==proc[k+1]]}
    sum(M.res)
}


n <- 0
for (i in 1:5) {

    ## select user interface
    urine[i,i] <- 1
    feces[i,i] <- 1
    wet[i,i] <- 1

    ## check first, if an interface produces a path
    if(i %in% c(2, 5)) { n.urine <- n.path(urine, options.urine) } else {n.urine <- 1}
    if(i %in% c(1, 2)) { n.feces <- n.path(feces, options.feces) } else {n.feces <- 1}
    if(i %in% c(3, 4, 5)) { n.wet <- n.path(wet, options.wet) } else {n.wet <- 1}

    urine[i,i] <- 0
    feces[i,i] <- 0
    wet[i,i] <- 0

    n <- n + n.urine * n.feces * n.wet
}

## print results
cat("Number of possible SanSys if everything is compatible:", n.max, "\n")
cat("Number of possible SanSys given the compatibility matrices:", n, "\n")
cat("That is a reduction to", round(n/n.max*100, 5), "percent. \n")


## =======================================================



